--
--	BigGuild
--
--	BigGuild is a replacement for the Guild panel of the Friends window. It uses a larger window,
--  and shows a bit more information.
--
--	Key functions in this file are:
--		BigGuild_ParseGuildRoster			Parse WoW's guild roster table, mirror it with an internal 'master' table
--		BigGuild_FilterGuildRoster			Create a filtered table of indexes into the master table
--		BigGuild_SortGuildRoster			Sort the filtered table
--		BigGuildStatus_Update				Update the UI
--
--	Each function in the list above calls the one below it as part of its purpose. UI Elements should be careful
--	to call the appropriate function, so we don't do more work than necessary (e.g. Column header tabs should call 
--	the sort function to re-sort the indexes--reparsing and filtering is not necessary).
--

--	Strings used in the KeyBindings window for a section devoted to our mod.
BINDING_HEADER_BIGUUILD_KEYBINDHEADERTITLE = "BigGuild Bindings";
BINDING_NAME_BIGGUILD_UITOGGLE = "Toggle BigGuild Window";

-- This is a table of tables that shadows Blizzards GuildRoster
-- The table is generated by BigGuild_ParseGuildRoster.
BigGuildRoster = {};

-- This is a table of indexes into BigGuildRoster. Elements of this table
-- are filtered and sorted.
BigGuildFilteredRoster = {};
BigGuild_CurrentSort = "name"
BigGuild_CurrentSortDirection = 1;
BigGuild_CurrentRankSelection = 1;

-- This is a table of names that are checked - that is, that the user has checkmarked.
BigGuildCheckedNamelist = {};

-- This is a constant table of names of character classes in WoW.
BigGuildClassNames = { "Druid", "Hunter", "Mage", "Paladin", "Priest", "Rogue", "Shaman", "Warlock", "Warrior", "Death Knight" };

function BigGuild_OnLoad(self)

	--Register the frame for the following events
--	self:RegisterEvent("ADDON_LOADED");
--	self:RegisterEvent("VARIABLES_LOADED");

	--Allows BigGuild to be closed with the Escape key
	tinsert(UISpecialFrames, "BigGuildFrame");
	
	-- Set up our slash command
	SlashCmdList["BIGGUILD"] = BigGuildWindow_Toggle;
	SLASH_BIGGUILD1 = "/bigguild";
	
	--Dragging involves some special registration
--	BigGuildFrame:RegisterForDrag("LeftButton");

	self:RegisterEvent("GUILD_ROSTER_UPDATE");
	self:RegisterEvent("RAID_ROSTER_UPDATE");
	
	--Initial sort is by name
	BigGuildNameColumnHeader:LockHighlight();
	
	--I couldnt sort out how to get the text to show so i created this
	BigGuild_FilterText = BigGuildNameColumnHeader:CreateFontString("BigGuild_FilterText" ,"ARTWORK","GameFontNormal");
	BigGuild_FilterText:SetPoint("Left")
	BigGuild_FilterText:SetJustifyH("LEFT")	--Left justified text	
	BigGuild_FilterText:SetText("|cffffffff   Name            Lvl    Class       Rank              Zone                  Raid") 
end

--Main event handler
function BigGuild_OnEvent()

	if ( event == "GUILD_ROSTER_UPDATE" ) then
		if BigGuildFrame:IsVisible() then
			if ( arg1 ) then
				GuildRoster();
			end
			BigGuild_ParseGuildRoster();
		end
	elseif event == "RAID_ROSTER_UPDATE" then
		if BigGuildFrame:IsVisible() then
			BigGuild_ParseGuildRoster();
		end
	end
end

-- Toggles the visibility of the main window
function BigGuildWindow_Toggle()
	if (BigGuildFrame:IsVisible()) then
		HideUIPanel(BigGuildFrame);
	else
		ShowUIPanel(BigGuildFrame);
		BigGuild_ParseGuildRoster();
	end
end

BigGuild_IsParsingGuildRoster = false;

--Queries Warcraft's built in guild roster and builds a private table of each guild member
-- Also calculates subtotals for number of members in each class, and calls BigGuild_FilterGuildRoster
-- to refilter the new roster. 
function BigGuild_ParseGuildRoster()
	local name, rank, rankIndex, level, class, zone, note, officernote, online, status;
	local inRaid;
	
	if not BigGuild_IsParsingGuildRoster then
		BigGuild_IsParsingGuildRoster = true;
		
		GuildRoster();
	
		--Check to see if we're in a guild...
		if not IsInGuild() then
			BigGuildMemberTotals:SetText("You are not in a guild.");
			return;
		end

		--Ensure we've got a good guild roster from the server
		BigGuildRoster = {};

		local showoffline = GetGuildRosterShowOffline();
		SetGuildRosterShowOffline(true);
		local numGuildMembers = GetNumGuildMembers(true);
		local numOnlineMembers = 0;
		
		local classTotals = {};
		local onlineTotals = {};
		local raidTotals = {};
		for index, className in ipairs(BigGuildClassNames) do
			classTotals[className] = 0;
			onlineTotals[className] = 0;
			raidTotals[className] = 0;
		end
		
		-- Make a local table of all the raid members
		local raidMembers = {};
		local raidMemberName;
		local numRaidMembers = GetNumRaidMembers();
		if numRaidMembers then
			for raidMember = 1, numRaidMembers do
				raidMemberName = GetRaidRosterInfo(raidMember); 
				raidMembers[raidMemberName] = raidMemberName;
			end
		end
		
		-- Iterate the list of guild members, get info on them, add them to the table, 
		-- generate various sums of characters.
		for i = 1, numGuildMembers, 1 do
			local raidBump = 0;
			local onlineBump = 0;
		
			-- Get the roster info for this member
			name, rank, rankIndex, level, class, zone, note, officernote, online, status = GetGuildRosterInfo(i);
			
			-- Is the member in the raid (if there is a raid...)
			if (numRaidMembers > 0 and raidMembers[name] ~= nil) then
				inRaid = "yes";
				raidBump = 1;
			else
				inRaid = " ";
			end
			
			-- Set up the online bump, and generate a "last online" string
			local lastOnlineString;
			if online then
				numOnlineMembers = numOnlineMembers + 1;
				onlineBump = 1;
				lastOnlineString = "online now";
			else
				lastOnlineString = BigGuild_GetLastOnline(i);
			end
			
			table.insert(BigGuildRoster, { name=name, level=level, class=class, rank=rank, rankIndex=rankIndex, 
					zone=zone, online=online, lastOnlineString=lastOnlineString, status=status,
					inRaid=inRaid, note=note, officernote=officernote });
					
			-- Do some summing on the members at this time as well
			classTotals[class] = classTotals[class] + 1;
			onlineTotals[class] = onlineTotals[class] + onlineBump;
			raidTotals[class] = raidTotals[class] + raidBump;
		end
				
		SetGuildRosterShowOffline(showoffline);
		
		-- Set the totals	
		for index, className in ipairs(BigGuildClassNames) do
			_G["BigGuild" .. className .. "ClassTotalsNumTotal"]:SetText(classTotals[className]);
			_G["BigGuild" .. className .. "ClassTotalsNumOnline"]:SetText(onlineTotals[className]);
			_G["BigGuild" .. className .. "ClassTotalsNumInRaid"]:SetText(raidTotals[className]);
		end
		BigGuildOverallClassTotalsNumTotal:SetText(numGuildMembers);
		BigGuildOverallClassTotalsNumOnline:SetText(numOnlineMembers);
		BigGuildOverallClassTotalsNumInRaid:SetText(numRaidMembers);

		--
		if numRaidMembers then
			BigGuildMemberTotals:SetText(numGuildMembers.." Members in guild, "..numOnlineMembers.." online, "..numRaidMembers.." in raid.");
		else
			BigGuildMemberTotals:SetText(numGuildMembers.." Members in guild, "..numOnlineMembers.." online.");
		end
		
		-- Parsing a new table means we need to go filter and sort again
		BigGuild_FilterGuildRoster();
		
		BigGuild_IsParsingGuildRoster = false;
	end
end

--Applies filters to BigGuild's private guild roster, creating a table of entries in the 
-- roster that pass the filters. Should be called whenever the filters change. Calculates
-- some more subtotals, and calls BigGuild_SortGuildRoster to sort the filtered entries.
function BigGuild_FilterGuildRoster()
	BigGuildFilteredRoster = {};
	
	-- Get the states of all the checkboxes
	local onlineFilter = BigGuildFilterOnlineCheckButton:GetChecked();
	local notOnlineFilter = BigGuildFilterNotOnlineCheckButton:GetChecked();
	local level60Filter = BigGuildFilterLevelCheckButton:GetChecked();
	local level70Filter = BigGuildFilterLevel70CheckButton:GetChecked();
	local inRaidFilter = BigGuildFilterInRaidCheckButton:GetChecked();
	local notInRaidFilter = BigGuildFilterNotInRaidCheckButton:GetChecked();
	local inZoneFilter = BigGuildFilterInZoneCheckButton:GetChecked();
	local notInZoneFilter = BigGuildFilterNotInZoneCheckButton:GetChecked();
	local isCheckedFilter = BigGuildFilterCheckedCheckButton:GetChecked();
	local rankFilter = BigGuildFilterRankCheckButton:GetChecked();
	local rankAboveFilter = BigGuildFilterRankAboveCheckButton:GetChecked();
	local minRank = UIDropDownMenu_GetSelectedID(BigGuildFilterRankDropDown) - 1;
	
	local isClassFilter = {};
	local numOfClass = {};
	local someClassChecked = false;
	for index, className in ipairs(BigGuildClassNames) do
		numOfClass[className] = 0;
		isClassFilter[className] = _G["BigGuildFilter" .. className .. "sCheckButton"]:GetChecked();
		if isClassFilter[className] then
			someClassChecked = true;
		end
	end
		
	-- If none of the class filters are enabled, all of them are enabled.
	if not someClassChecked then
		for index, className in ipairs(BigGuildClassNames) do
			isClassFilter[className] = true;
		end
	end
	
	-- Get the player's current zone
	local playerZoneName = GetZoneText();
			
	local numMembers = table.getn(BigGuildRoster);
	for i = 1, numMembers, 1 do
		local curMember = BigGuildRoster[i];
		local addMember = true;
		
		--This whole thing can be shortened into one garguantan if statement.
		if  not ((onlineFilter ~= nil and not curMember.online) or
				(notOnlineFilter and curMember.online) or
				(level60Filter and curMember.level < 80) or
				(level70Filter and curMember.level < 85) or
				(inRaidFilter and curMember.inRaid ~= "yes") or
				(notInRaidFilter and curMember.inRaid == "yes") or
				(inZoneFilter and curMember.zone ~= playerZoneName) or
				(notInZoneFilter and curMember.zone == playerZoneName) or
				(isCheckedFilter and BigGuildCheckedNamelist[curMember.name] == nil) or
				(rankFilter and (curMember.rankIndex > minRank or
					(not rankAboveFilter and curMember.rankIndex < minRank))) or
				(not isClassFilter[curMember.class])) then
								
			-- Passed all filters, so add it to the list
			table.insert(BigGuildFilteredRoster, i);
			
			numOfClass[curMember.class] = numOfClass[curMember.class] + 1;
		end
	end
		
	-- Set the totals for the "Shown" column
	for index, className in ipairs(BigGuildClassNames) do
		_G["BigGuild" .. className .. "ClassTotalsNumShown"]:SetText(numOfClass[className]);
	end
	BigGuildOverallClassTotalsNumShown:SetText(table.getn(BigGuildFilteredRoster));
		
	--Changing the filter means we need to re-sort...
	BigGuild_SortGuildRoster();
end

--Sorts the filtertable according to the currently hilighted sort column. Called by
-- the click handlers for the column titles, and by functions that change the guild roster
-- or the member filters. Changing the sort (or the filters, or the roster) necessitates
-- a call to BigGuildStatus_Update.
function BigGuild_SortGuildRoster()
	table.sort(BigGuildFilteredRoster, BigGuildSortComparator);

	--Changing the sort requires updating the display
	BigGuildStatus_Update();
end

--Redraws the guild membertable in response to a change. 
function BigGuildStatus_Update()
	
	local name, rank, rankIndex, level, class, zone, note, officernote, online, status;
	local guildOffset = FauxScrollFrame_GetOffset(BigGuildListScrollFrame);
	
	for i=1, 32, 1 do
		local index = guildOffset + i;
		local memberTable = nil;
		local memberTableIndex = BigGuildFilteredRoster[index];
		if memberTableIndex then
			memberTable = BigGuildRoster[memberTableIndex];
		end
		
		local button = getglobal("BigGuildTableRow"..i);
		local checkbox = getglobal("BigGuildTableRow"..i.."CB");
		
		if (memberTable ~= nil) then
			getglobal("BigGuildTableRow"..i.."Name"):SetText(memberTable["name"]);
			getglobal("BigGuildTableRow"..i.."Level"):SetText(memberTable["level"]);
			getglobal("BigGuildTableRow"..i.."Class"):SetText(memberTable["class"]);
			getglobal("BigGuildTableRow"..i.."Rank"):SetText(memberTable["rank"]);
			getglobal("BigGuildTableRow"..i.."Zone"):SetText(memberTable["zone"]);
			getglobal("BigGuildTableRow"..i.."Raid"):SetText(memberTable["inRaid"]);
			
			if ( memberTable["online"] ) then
				getglobal("BigGuildTableRow"..i.."Name"):SetTextColor(1.0, 0.82, 0.0);
				getglobal("BigGuildTableRow"..i.."Level"):SetTextColor(1.0, 1.0, 1.0);
				getglobal("BigGuildTableRow"..i.."Class"):SetTextColor(1.0, 1.0, 1.0);
				getglobal("BigGuildTableRow"..i.."Rank"):SetTextColor(1.0, 1.0, 1.0);
				getglobal("BigGuildTableRow"..i.."Zone"):SetTextColor(1.0, 1.0, 1.0);
				getglobal("BigGuildTableRow"..i.."Raid"):SetTextColor(1.0, 1.0, 1.0);
			else
				getglobal("BigGuildTableRow"..i.."Name"):SetTextColor(0.5, 0.5, 0.5);
				getglobal("BigGuildTableRow"..i.."Level"):SetTextColor(0.5, 0.5, 0.5);
				getglobal("BigGuildTableRow"..i.."Class"):SetTextColor(0.5, 0.5, 0.5);
				getglobal("BigGuildTableRow"..i.."Rank"):SetTextColor(0.5, 0.5, 0.5);
				getglobal("BigGuildTableRow"..i.."Zone"):SetTextColor(0.5, 0.5, 0.5);
				getglobal("BigGuildTableRow"..i.."Raid"):SetTextColor(0.5, 0.5, 0.5);
			end
			
			checkbox:SetChecked(BigGuildCheckedNamelist[memberTable["name"]] ~= nil);
			
			button:Show();
			checkbox:Show();
			button.guildIndex = memberTableIndex;
		else
			button:Hide();
			checkbox:Hide();
			button.guildIndex = 0;
		end

	end
	
	-- ScrollFrame update
	--FauxScrollFrame_Update(	frame, 						numItems, 					numToDisplay, 	valueStep, button, smallWidth, bigWidth, highlightFrame, smallHighlightWidth, bigHighlightWidth, alwaysShowScrollBar )
	FauxScrollFrame_Update(		BigGuildListScrollFrame, 	#BigGuildFilteredRoster, 	30, 			15 			);

end

function BigGuildSortComparator(a, b)
	local aMember = BigGuildRoster[a];
	local bMember = BigGuildRoster[b];
	
	local aSortKey = aMember[BigGuild_CurrentSort];
	local bSortKey = bMember[BigGuild_CurrentSort];
	if aSortKey == nil then
		return true;
	end
	if bSortKey == nil then
		return false;
	end
	
	if BigGuild_CurrentSortDirection == 1 then
		return aSortKey < bSortKey;
	else
		return  aSortKey > bSortKey;
	end
end

--Sets the sort type and sort direction for the guild roster table
function BigGuild_SetRosterSort(sortType)
	if (BigGuild_CurrentSort == sortType) then
		if BigGuild_CurrentSortDirection == 1 then
			BigGuild_CurrentSortDirection = 0;
		else
			BigGuild_CurrentSortDirection = 1;
		end
	else
		BigGuild_CurrentSort = sortType;
		BigGuild_CurrentSortDirection = 1;
	end
	BigGuild_SortGuildRoster();
end

--Clears all highlighting on the guild member table column headers
function BigGuild_DehilightColumnHeaders()
	BigGuildNameColumnHeader:UnlockHighlight();
	BigGuildLevelColumnHeader:UnlockHighlight();
	BigGuildClassColumnHeader:UnlockHighlight();
	BigGuildRankColumnHeader:UnlockHighlight();
	BigGuildZoneColumnHeader:UnlockHighlight();
	BigGuildInRaidColumnHeader:UnlockHighlight();
end

--Sets the column widths and sort keys
function BigGuildColumnHeader_OnLoad(self, width, sortKey)
	self:SetWidth(width);
	getglobal(self:GetName().."Middle"):SetWidth(width - 9);
	self.sortType = sortKey;
end

function BigGuildFilterRankDropDown_Initialize()
	local info;
	
	if (IsInGuild()) then
		local info;
		local numRanks = GuildControlGetNumRanks();
		for i=1, numRanks, 1 do
			info = {};
			info.text = GuildControlGetRankName(i);
			info.func = BigGuildFilterRankDropDownButton_OnClick;
			UIDropDownMenu_AddButton(info);
		end
		if BigGuild_CurrentRankSelection > numRanks then
			BigGuild_CurrentRankSelection = numRanks;
		end
	else
		info = {};
		info.text = " ";
		UIDropDownMenu_AddButton(info);
		BigGuild_CurrentRankSelection = 1;
	end
	
	UIDropDownMenu_SetSelectedID(BigGuildFilterRankDropDown, BigGuild_CurrentRankSelection);
end

function BigGuildFilterRankDropDown_OnLoad()
--	UIDropDownMenu_Initialize(this, BigGuildFilterRankDropDown_Initialize);
	UIDropDownMenu_SetWidth(BigGuildFilterRankDropDown, 80);
	UIDropDownMenu_SetButtonWidth(BigGuildFilterRankDropDown, 24);
	UIDropDownMenu_JustifyText(BigGuildFilterRankDropDown, "LEFT")
end

function BigGuildFilterRankDropDownButton_OnClick(self)
	UIDropDownMenu_SetSelectedID(BigGuildFilterRankDropDown, self:GetID());
	BigGuild_FilterGuildRoster();
end

function BigGuildMemberList_OnEnter(self)
	local memberTableIndex = self.guildIndex;
	local memberTable;
	if memberTableIndex then
		memberTable = BigGuildRoster[memberTableIndex];
		BigGuildDetailMemberName:SetText(memberTable["name"]);
		BigGuildDetailLastOnline:SetText(memberTable["lastOnlineString"]);
		BigGuildMemberNote_Label:SetText("Note: " .. memberTable["note"]);
		BigGuildOfficerNote_Label:SetText("Officer Note: " .. memberTable["officernote"]);
	end
end

function BigGuildToggleMemberCheckmark(checkBoxIndex)
	local checkbox = getglobal("BigGuildTableRow"..checkBoxIndex.."CB");
	local isChecked = checkbox:GetChecked();
	local guildOffset = FauxScrollFrame_GetOffset(BigGuildListScrollFrame);
	local memberTable = nil;
	local memberTableIndex = BigGuildFilteredRoster[guildOffset + checkBoxIndex];
	if memberTableIndex then
		memberTable = BigGuildRoster[memberTableIndex];
	end
	
	local memberName = memberTable.name;
	
	if BigGuildCheckedNamelist[memberName] then
		BigGuildCheckedNamelist[memberName] = nil;
	else
		BigGuildCheckedNamelist[memberName] = memberName;
	end

	if isChecked then
		PlaySound("igMainMenuOptionCheckBoxOff");
	else
		PlaySound("igMainMenuOptionCheckBoxOn");
	end

end

-- Stolen wholesale from FriendsFrame.lua
function BigGuild_GetLastOnline(guildIndex)
	local year, month, day, hour = GetGuildRosterLastOnline(guildIndex);
	local lastOnline;
	if ( (year == 0) or (year == nil) ) then
		if ( (month == 0) or (month == nil) ) then
			if ( (day == 0) or (day == nil) ) then
				if ( (hour == 0) or (hour == nil) ) then
					lastOnline = LASTONLINE_MINS;
				else
					lastOnline = format(GetText("LASTONLINE_HOURS", nil, hour), hour);
				end
			else
				lastOnline = format(GetText("LASTONLINE_DAYS", nil, day), day);
			end
		else
			lastOnline = format(GetText("LASTONLINE_MONTHS", nil, month), month);
		end
	else
		lastOnline = format(GetText("LASTONLINE_YEARS", nil, year), year);
	end
	return lastOnline;
end

